#include "Vector.h"

// Konstruktor
Vector::Vector(size_t len, double val) 
	: len(len)
	, data(new double[len]) 
{
	#ifdef VECTOR_DEBUG
		std::cout << "CTOR ";
	#endif
	for (size_t i = 0; i < len; ++i)
		data[i] = val;
}

// Másoló konsruktor
Vector::Vector(Vector const& v)
	: len(v.len)
	, data(new double[v.len])
{
	#ifdef VECTOR_DEBUG
		std::cout << "COPY ";
	#endif
	
	for (size_t i = 0; i < len; ++i)
		data[i] = v.data[i];
}

// Értékadás operátor
Vector &Vector::operator=(Vector const& v)
{
	#ifdef VECTOR_DEBUG
		std::cout << "OP= ";
	#endif
	
	if (len != v.len)
		throw "operator=() : Size mismatch!";
	
	for (size_t i = 0; i < len; ++i)
		data[i] = v.data[i];
	
	return *this;
}

// Destruktor
Vector::~Vector()
{
	#ifdef VECTOR_DEBUG
		std::cout << "DTOR ";
	#endif
	
	delete [] data;
}

// Indexelés (const)
double const &Vector::operator[](size_t idx) const
{
	if (idx >= len)
		throw "operator[] const() : Bad index!";
	
	return data[idx];
}

// Indexelés (nem const)
double &Vector::operator[](size_t idx)
{
	if (idx >= len)
		throw "operator[]() : Bad index!";
	
	return data[idx];
}

// Vektorok összeadása
Vector Vector::operator+(Vector const &v) const
{
	#ifdef VECTOR_DEBUG
		std::cout << "OP+ ";
	#endif
	
	if (this->len != v.len)
		throw "operator+() : Size mismatch!";
	
	// Készítünk egy új vektort
	Vector ret(this->len);
	// Kitöltjük az elemeket
	for (size_t i = 0; i < len; ++i)
		ret.data[i] = this->data[i] + v.data[i];
	
	// Visszatérés
	return ret;
}

// Inserter operátor
std::ostream &operator<<(std::ostream &os, Vector const &v)
{
	size_t len = v.getLen();
	os << '[' << len << "] = {";
	for (size_t i = 0; i < len; ++i)
	{
		os << v[i]; 
		if (i != len-1)
			os << ", ";
	}
	os << '}';
	
	return os;
}
